import androidx.compose.animation.ExperimentalSharedTransitionApi
import androidx.compose.animation.SharedTransitionLayout
import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.grid.GridCells
import androidx.compose.foundation.lazy.grid.LazyVerticalGrid
import androidx.compose.foundation.lazy.grid.items
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.material.Button
import androidx.compose.material.Card
import androidx.compose.material.ExperimentalMaterialApi
import androidx.compose.material.FilterChip
import androidx.compose.material.OutlinedTextField
import androidx.compose.material.Scaffold
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.compose.ui.window.Dialog
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import details.RecipeDetails
import model.Recipe
import model.americanList
import model.exampleList
import model.exampleTags
import recipeslist.RecipesListScreen
import sensor.SensorManager
import ui.theme.MainTheme


enum class Theme {
    Auto,
    Light,
    Dark
}

@OptIn(ExperimentalSharedTransitionApi::class)
@Composable
fun App(sensorManager: SensorManager?, isLarge: Boolean = false) {

    val navController = rememberNavController()
    var theme by remember { mutableStateOf(Theme.Auto) }

    val onThemeToggle = {
        theme = when (theme) {
            Theme.Auto -> Theme.Light
            Theme.Light -> Theme.Dark
            Theme.Dark -> Theme.Auto
        }
    }
    val isDarkTheme: Boolean? = when (theme) {
        Theme.Dark -> true
        Theme.Light -> false
        Theme.Auto -> null
    }

    MainTheme(isDarkTheme ?: isSystemInDarkTheme()) {

        var show by remember { mutableStateOf(false) }
        var searchBar by remember {mutableStateOf(true)}
        var search by remember { mutableStateOf("") }
        val tags = remember { mutableStateListOf<String>() }
        var recipeTags by remember {mutableStateOf(mutableListOf<String>())}

        for (recipe in getRecipeList())
        {
            for(tag in recipe.tags)
            {
                if(!recipeTags.contains(tag))
                    recipeTags.add(tag)
            }
        }


        if(show){
            filterCard(recipeTags) {
                tags.removeAll(tags)
                tags.addAll(it)
                show = false }
        }

        val filteredItems = getFilteredRecipeList(tags, search)
        var currentRecipe = getRecipeList().first()

        if(filteredItems.isNotEmpty())
            currentRecipe = filteredItems.first()

        Scaffold {
            Column {
                if(searchBar) {
                    Row {
//                        Button(
//                            onClick = {
//                                onThemeToggle.invoke()
//                            }
//                        ) {
//                            Text(text = theme.name)
//                        }
                        Button(
                            onClick = {
                                show = true
                            },
                        ) {
                            Text(text = "Filter")
                        }
                        InputFieldState(
                            value = search,
                            label = "Search",
                            modifier = Modifier
                                .width(300.dp)
                        ) {
                            search = it
                        }
                    }
                }

                SharedTransitionLayout {
                    val sharedTransitionScope = this
                    NavHost(
                        navController = navController,
                        startDestination = RecipeAppScreen.List.name,
                        modifier = Modifier.fillMaxSize()
                    ) {
                        composable(route = RecipeAppScreen.List.name) {
                            RecipesListScreen(animatedVisibilityScope = this,
                                sharedTransactionScope = sharedTransitionScope,
                                isLarge = isLarge,
                                items = filteredItems,
                                onClick = { recipe ->
                                    searchBar = false
                                    currentRecipe = recipe
                                    navController.navigate(RecipeAppScreen.Details.name)
                                })
                        }
                        composable(route = RecipeAppScreen.Details.name) {
                            RecipeDetails(animatedVisibilityScope = this,
                                sharedTransactionScope = sharedTransitionScope,
                                isLarge = isLarge,
                                sensorManager = sensorManager,
                                recipe = currentRecipe,
                                goBack = {
                                   searchBar = true
                                   navController.popBackStack()
                                })
                        }
                    }
                }
            }
        }
    }
}


@OptIn(ExperimentalMaterialApi::class)
@Composable
fun filterCard(
    recipeTags: List<String>,
    onDismissRequest : (List<String>) -> Unit,
) {

    val tags by remember { mutableStateOf(recipeTags) }
    var tagsFiltered: List<String> = listOf()

    Dialog(onDismissRequest = {

        onDismissRequest(tagsFiltered)

    }) {
        Card {
            LazyVerticalGrid(
                columns = GridCells.Adaptive(minSize = 128.dp)
            ) {
                items(tags) { tag ->
                    var selected by remember { mutableStateOf(false) }
                    FilterChip(
                        selected = selected,
                        onClick = {
                            selected = !selected
                            tagsFiltered = if(selected) {
                                tagsFiltered.plus(tag)
                            } else{
                                tagsFiltered.minus(tag)
                            }
                                  },
                        content = { Text(tag) }
                    )
                }
            }
        }
    }
}

enum class RecipeAppScreen {
    List, Details,
}

fun List<String>.containsPartial(text:String) : Boolean {
    return  this.any {it.contains(text)}
}

@Composable
fun getFilteredRecipeList(
    tags : List<String>,
    search : String,
) : List<Recipe> {
    val items = getRecipeList()

    var recipes = items.filter { it.tags.containsAll(tags)}

    recipes = recipes.filter {
        it.title.contains(search) || it.ingredients.containsPartial(search)
    }

    return recipes
}

@Composable
fun getRecipeList() : List<Recipe> {
    val exampleRecipes by remember { mutableStateOf(exampleList) }
    val americanRecipes by remember { mutableStateOf(americanList) }
    val items : MutableList<Recipe> = mutableListOf()


    items.addAll(exampleRecipes)
    items.addAll(americanRecipes)

    return items
}

@Composable
fun InputFieldComponent(
    text: String,
    onChange: (String) -> Unit,
    modifier: Modifier = Modifier,
    singleLine: Boolean = true,
    label: String = "Some val",
    keyboardActions: KeyboardActions = KeyboardActions.Default,
) {
    OutlinedTextField(
        value = text,
        onValueChange = onChange,
        label = { Text(text = label)},
        modifier = modifier,
        singleLine = singleLine,
        keyboardActions = keyboardActions,
    )
}


@Composable
fun InputFieldState(
    value: String,
    label: String,
    modifier: Modifier,
    keyboardActions: KeyboardActions = KeyboardActions.Default,
    onVal: (String) -> Unit,
) {

    Column(
        modifier = Modifier
    ) {
        InputField(value, label, modifier, onVal, keyboardActions)
        Spacer(modifier = Modifier.padding(10.dp))
    }
}

@Composable
fun InputField(
    text: String,
    label: String,
    modifier: Modifier,
    onValChange: ((String) -> Unit)?,
    keyboardActions: KeyboardActions,
) {
    if (onValChange != null) {
        InputFieldComponent(
            text = text,
            onChange = onValChange,
            label = label,
            modifier = modifier,
            keyboardActions = keyboardActions,
        )
    }
}